if exists (select * from sys.objects where name = 'scheduler_health_monitor_xml')
	drop table scheduler_health_monitor_xml
go

select *, convert(XML, record) as xml_record
into scheduler_health_monitor_xml
from sys.dm_os_ring_buffers
where ring_buffer_type = 'RING_BUFFER_SCHEDULER_MONITOR'
go

declare @node int
declare @scheduler int
declare @beginyields int
declare @endyields int

select 
	@node = n.value ('(Node)[1]', 'int'),
	@scheduler = n.value ('(Scheduler)[1]', 'int'),
	@beginyields = n.value ('(Yields)[1]', 'int')
from 
	scheduler_health_monitor_xml cross apply xml_record.nodes('/Record/SchedluerMonitorEvent/NonYieldBegin') non_yield(n)
where 
	xml_record.exist ('(/Record/SchedluerMonitorEvent/NonYieldBegin)') = 1

select 
	@endyields = n.value ('(Yields)[1]', 'int')
from 
	scheduler_health_monitor_xml cross apply xml_record.nodes('/Record/SchedluerMonitorEvent/NonYieldEnd') non_yield(n)
where 
	xml_record.exist ('(/Record/SchedluerMonitorEvent/NonYieldEnd)') = 1
	and n.exist ('Node[(text()[1] cast as xs:int?)=sql:variable("@node")]') = 1
	and	n.exist ('Scheduler[(text()[1] cast as xs:int?)=sql:variable("@scheduler")]') = 1
	and	n.exist ('Yields[(text()[1] cast as xs:int?)>=sql:variable("@beginyields")]') = 1

select 
	'BEGIN' as State, 
	@node as NodeId,
	@scheduler as SchedulerId,
	@beginyields as NumYields,
	n.value ('(Worker)[1]', 'varbinary (20)') as Worker,
	n.value ('(WorkerUtilization)[1]', 'int') as WorkerUtil,
	n.value ('(ProcessUtilization)[1]', 'int') as ProcessUtil,
	n.value ('(SystemIdle)[1]', 'int') as SystemIdle,
	n.value ('(UserModeTime)[1]', 'int')/10000.0 as UserModeTime,
	n.value ('(KernelModeTime)[1]', 'int')/10000.0 as KernelModeTime,
	n.value ('(PageFaults)[1]', 'int') as PageFaults,
	n.value ('(WorkingSetDelta)[1]', 'int') as WorkingSetDelta,
	n.value ('(MemoryUtilization)[1]', 'int') as MemoryUtilPct
from 
	scheduler_health_monitor_xml cross apply xml_record.nodes('/Record/SchedluerMonitorEvent/NonYieldBegin') non_yield(n)
where 
	n.exist ('Node[(text()[1] cast as xs:int?)=sql:variable("@node")]') = 1
	and	n.exist ('Scheduler[(text()[1] cast as xs:int?)=sql:variable("@scheduler")]') = 1
	and	n.exist ('Yields[(text()[1] cast as xs:int?)=sql:variable("@beginyields")]') = 1

UNION

select 
	'END' as State, 
	@node as NodeId,
	@scheduler as SchedulerId,
	@endyields as NumYields,
	n.value ('(Worker)[1]', 'varbinary (20)') as Worker,
	n.value ('(WorkerUtilization)[1]', 'int') as WorkerUtil,
	n.value ('(ProcessUtilization)[1]', 'int') as ProcessUtil,
	n.value ('(SystemIdle)[1]', 'int') as SystemIdle,
	n.value ('(UserModeTime)[1]', 'int')/10000.0 as UserModeTime,
	n.value ('(KernelModeTime)[1]', 'int')/10000.0 as KernelModeTime,
	n.value ('(PageFaults)[1]', 'int') as PageFaults,
	n.value ('(WorkingSetDelta)[1]', 'int') as WorkingSetDelta,
	n.value ('(MemoryUtilization)[1]', 'int') as MemoryUtilPct
from 
	scheduler_health_monitor_xml cross apply xml_record.nodes('/Record/SchedluerMonitorEvent/NonYieldEnd') non_yield(n)
where 
	n.exist ('Node[(text()[1] cast as xs:int?)=sql:variable("@node")]') = 1
	and	n.exist ('Scheduler[(text()[1] cast as xs:int?)=sql:variable("@scheduler")]') = 1
	and	n.exist ('Yields[(text()[1] cast as xs:int?)=sql:variable("@endyields")]') = 1



